/*
	bcomlib.c
	external functions for bunix commands
	Ver. 0.5
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

/*
	ftype: return a letter to identify the file-type
*/
char ftype(char *path)
{
	struct stat buf;

	if (stat(path, &buf) == 0)
	{
		switch(buf.st_mode & S_IFMT)
		{
			case S_IFDIR:	/* dir */
				return('d');
			case S_IFREG:	/* file */
				return('-');
			case S_IFCHR:	/* character device */
				return('c');
			case S_IFBLK:	/* block device */
				return('b');
			case S_IFIFO:	/* pipe */
				return('p');
			default:
				return('?');
		}
	}
	else
		return('n');
}

/*
	fino: return inode number or -1 on failure
*/
int fino(const char *path)
{
	struct stat buf;

	if (stat(path, &buf) == 0)
	{
		return(buf.st_ino);
	}
	else
	{
		return(-1);
	}
}

/* fatime: return last access time */
time_t fatime(const char *path)
{
	struct stat buf;

	if (stat(path, &buf) == 0)
	{
		return(buf.st_atime);
	}
	else
	{
		return(-1);
	}
}

/* fmtime: return last modification time */
time_t fmtime(const char *path)
{
	struct stat buf;

	if (stat(path, &buf) == 0)
	{
		return(buf.st_mtime);
	}
	else
	{
		return(-1);
	}
}
