/*
	bsh: Bruno's shell
	Ver. 0.5
	-v	The shell shall write its input to stderr as it is read.
	-x	The shell shall write to stderr a trace for each command
		after it expands the command and before it executes it.
TO DO
	execLine function
	dot command
	file redirection
	read command
	if command
	for command
	while command
*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#define MAXLIN	256
#define VAR_SIZE	160

#define	EXIT	0
#define	REM	1
#define	CHDIR	2
#define SYSTEM	3
#define SET	4
#define LET	5
#define EXPORT	6

char *prog="bsh";
int CLargc=0;		/* argument counter in current line */
char CLargv[64][64];	/* arguments in current line */
char lineb[MAXLIN];
char locVar[80][VAR_SIZE];
int cLocVar=0;
extern char **environ;

int opv=0;
int opx=0;

/* varName: return variable name or NULL */
char *varName(char *string)
{
	char *p=NULL;
	char name[32];
	int i;

	for (i=0; i<strlen(string) && string[i]!='='; i++)
	{
		name[i]=string[i];
	}
	name[i]='\0';
	p=name;

	return p;
}

/* varValue: return variable value or NULL */
char *varValue(char *string)
{
	char *p=NULL;
	char value[VAR_SIZE];
	int i, j;

	for (i=0; i<strlen(string) && string[i]!='='; i++)
		;
	++i;
	for (j=0; i<strlen(string); i++, j++)
	{
		value[j]=string[i];
	}
	value[j]='\0';
	p=value;

	return p;
}

/* findVar: return a pointer to a stored variable or NULL */
char *findVar(char *string)
{
	char *p=NULL;
	int i;

	for (i=0; i<cLocVar; i++)
	{
		if (strcmp(string, varName(locVar[i])) == 0)
		{
			p=locVar[i];
			return p;
		}
	}

	for (i=0; environ[i][0] != NULL; i++)
	{
		if (strcmp(string, varName(environ[i])) == 0)
		{
			p=environ[i];
			return p;
		}

	}

	return p;
}

void usage(int x)
{
	switch(x)
	{
	case EXIT:
		fprintf(stderr,"usage: %s [-xv]|[+xv]\n", prog);
		exit(1);
		break;
	case SET:
		fprintf(stderr,"%s: usage: set [-xv]|[+xv]\n", prog);
		break;
	case EXPORT:
		fprintf(stderr,"%s: usage: export VAR1 [ ... VARN ]\n",prog);
		break;
	case LET:
		fprintf(stderr,"%s: usage: VAR_NAME=value\n", prog);
		break;
	default:
		fprintf(stderr,"%s: error unknown\n", prog);
	}

}

void optCheck(char *opt, int x)
{
	int i;
	int state=0;

	switch (opt[0])
	{
	case '-':
		for (i=1; i<strlen(opt); i++)
		{
			switch(opt[i])
			{
			case 'v':
				opv=1;
				break;
			case 'x':
				opx=1;
				break;
			default:
				++state;
				if (x == EXIT)
					usage(x);
			}
		}
		break;
	case '+':
		for (i=1; i<strlen(opt); i++)
		{
			switch(opt[i])
			{
			case 'v':
				opv=0;
				break;
			case 'x':
				opx=0;
				break;
			default:
				++state;
				if (x == EXIT)
					usage(x);
			}
		}
	}
	if (state>0)
		usage(x);
}

int parse(char *linea)
{
	int i=0, j=0, y=0;
	int inVar=0;
	char tmparg[MAXLIN];
	char tmpVar[32];
	char *value;
	int quoting=0;

	for (i=0; linea[i]==' ' || linea[i]=='\t'; i++)
		;
	for ( ; i<=strlen(linea); i++)
	{
		if (inVar)
		{
			if (isalnum(linea[i]))
			{
				tmpVar[y]=linea[i];
				++y;
			}
			else
			{
				tmpVar[y]='\0';
				value=varValue(findVar(tmpVar));
				for (y=0; y<=strlen(value); y++)
				{
					lineb[j]=value[y];
					++j;
				}
				lineb[--j]=linea[i];
				++j;
				inVar=0;
				y=0;
			}
		}
		else
		{
			if (linea[i] == '$' && isalpha(linea[i+1])!=0 &&
				linea[i-1]!='\\')
			{
				inVar=1;
			}
			else
			{
				lineb[j]=linea[i];
				++j;
			}
		}
	}

	if (lineb[strlen(lineb)-1] == '\n')
		lineb[strlen(lineb)-1] = '\0';

	/* building tokens */
	for (i=0, j=0, CLargc=0; i<=strlen(lineb); i++)
	{
		if (quoting)
		{
			if (lineb[i]=='\"')
			{
				tmparg[j]='\"';
				tmparg[j+1]='\0';
				strcpy(CLargv[CLargc], tmparg);
				j=0;
				++CLargc;
				quoting=0;
			}
			else
			{
				tmparg[j]=lineb[i];
				++j;
			}
		}
		else
		{
			if (j == 0 && (lineb[i]==' ' || lineb[i]=='\t'))
				;
			else
			{
				if (lineb[i]=='\"')
				{
					quoting=1;
					// tmparg[j]=lineb[i];
					// ++j;
				}
				if (lineb[i]==' ' || lineb[i]=='\t' ||
					lineb[i]=='\n' || lineb[i]=='\0' ||
					quoting)
				{
					tmparg[j]='\0';
					strcpy(CLargv[CLargc], tmparg);
					j=0;
					++CLargc;
					if (quoting)
					{
						tmparg[j]=lineb[i];
						++j;
					}
				}
				else
				{
					tmparg[j]=lineb[i];
					++j;
				}
			}
		}
	}

#ifdef TEST
	printf("lineb=(%s)\n", lineb);
	printf("CLargc=%d\n", CLargc);
	for (i=0; i<CLargc; i++)
	{
		printf("CLargv[%d]=\"%s\"\n", i, CLargv[i]);
	}
#endif

	if (lineb[0]==0 || lineb[0]=='#' || lineb[0]=='\r' || lineb[0]=='\n')
		return(REM);
	if ((lineb[0]==4) || (strncmp(lineb,"exit",4)==0))
		return(EXIT);
	if (strcmp(CLargv[0],"cd")==0)
		return(CHDIR);
	if (strcmp(CLargv[0],"set")==0)
		return(SET);
	if (strcmp(CLargv[0],"export")==0)
		return(EXPORT);
	if (strchr(CLargv[0], '=') != NULL)
		return(LET);

	return(SYSTEM);
}

int let(char *string)
{
	int i;

	for (i=0; i<cLocVar; i++)
	{
		if (strcmp(varName(string), varName(locVar[i])) != NULL)
		{
			strcpy(locVar[i], string);
			return 0;
		}
	}
	strcpy(locVar[cLocVar], string);
	++cLocVar;
	return 0;
}

int set(char *linea)
{
	int i=0;
	char opt[MAXLIN];

	if (CLargc == 1)
	{
		while(environ[i][0] != NULL)
		{
			printf("%d (%s)\n", i, environ[i]);
			++i;
		}
	}
	else
	{
		strcpy(opt, CLargv[1]);
		optCheck(opt, SET);
	}

	return 0;
}

int export()
{
	int i;

	if (CLargc < 2)
	{
		usage(EXPORT);
		return 1;
	}

	for (i=1; i<CLargc; i++)
	{
		printf("(%s)(%s)\n", CLargv[i], findVar(CLargv[i]));
		putenv(findVar(CLargv[i]));
	}

	return 0;
}

int main(int argc, char *argv[])
{
	char space=' '; /* a char but not a backslash before linea is needed
			   by parse() */
	char linea[MAXLIN], argomento[MAXLIN];
	int online = 0;
	int comando = REM;
	int i, j;
	FILE *fi;
	char opt[32];
	int wOpt=0;

	/* options check */
	for (i=1; i<argc; i++)
	{
		strcpy(opt, argv[i]);
		if (opt[0]=='-' || opt[0]=='+')
		{
			++wOpt;
			optCheck(opt, EXIT);
		}
	}

	if (argc == 1+wOpt)
		online = 1;
	else
	{
		if (fi = fopen(argv[1+wOpt],"r"))
		{
			online = 0;
		}
		else
		{
			fprintf(stderr,"bsh: cannot open %s\n",
				argv[1]);
			exit(1);
		}
	}

	while (comando != EXIT)
	{
		if (online)
		{
			printf("$ ");
			gets(linea);
		}
		else
		{
			if (fgets(linea,MAXLIN,fi) == NULL)
			{
				comando = EXIT;
			}
		}

		if (opv)
		{
			fprintf(stderr, "%s\n", linea);
			fflush(stderr);
		}

		if (comando != EXIT)
			comando=parse(linea);

		if (opx)
		{
			fprintf(stderr, "+ %s\n", lineb);
			fflush(stderr);
		}

		switch(comando)
		{
		case SYSTEM:
			system(lineb);
			break;
		case REM:
		case EXIT:
			break;
		case CHDIR:
			if (CLargc == 1)
				printf("%s: cd: argument required\n",
					prog);
			else
			{
				if (chdir(CLargv[1]) == -1)
					printf("%s: cannot change dir to %s\n",
						prog, CLargv[1]);
			}
			break;
		case SET:
			set(lineb);
			break;
		case LET:
			if (CLargv[0][strlen(CLargv[0])-1]=='=')
			{
				if (CLargc > 1)
					let(strcat(CLargv[0],CLargv[1]));
				else
					usage(LET);
			}
			else
			{
				if (CLargc == 1)
					let(CLargv[0]);
				else
					usage(LET);
			}
			break;
		case EXPORT:
			export();
			break;
		default:
			break;
		}
	}
	if (!online)
		fclose(fi);
	return(0);
}
