/*
	ls.c
	list directory content
	Ver. 0.5
TO DO:
	Sort by name.
	Sort by time.
	Reverse sort.
	Check POSIX compatibility.
*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include "bcom.h"

#define NAME_MAX	32
#define MAX_PATH	1024

char *prog="ls";

int op1=0;
int opa=0;
int opc=1;
int opl=0;

void usage()
{
	printf("usage: %s [-c|l|1][a] [file1 ... fileN]\n", prog);
	exit(1);
}

/* fsize: return file size */
long int fsize(char *name)
{
	struct stat stbuf;

	if (stat(name, &stbuf) == -1)
	{
		fprintf(stderr,"%s: cannot stat %s\n", prog, name);
		return -1;
	}

	return stbuf.st_size;
}

int listFile(char *fileName, char *dName)
{
	char	strTime[64];
	time_t	mtime, *ptime, now;

	now=time(ptime);
	mtime=fmtime(fileName);
	ptime=&mtime;

	if (opl)
	{
		sprintf(strTime, "%s", ctime(ptime));
		if (now-mtime > 365*24*60*60)
		{
			strTime[11]='\0';
			strTime[24]='\0';
			printf("%c %8d %s%s %s\n", ftype(fileName),
				fsize(fileName), strTime+4, strTime+19,
				dName);
		}
		else
		{
			strTime[16]='\0';
			printf("%c %8d %s %s\n", ftype(fileName),
				fsize(fileName), strTime+4, dName);
		}
	}
	if (op1)
		printf("%s\n", dName);
	if (opc)
		printf("%s\t", dName);

	return 0;
}

int listDir(char *dirName)
{
	char name[MAX_PATH];
	struct dirent *dp;
	DIR *dfd;

	if ((dfd=opendir(dirName)) == NULL)
	{
		fprintf(stderr,"%s: cannot open %s\n", prog, dirName);
		return +1;
	}
	while ((dp=readdir(dfd)) != NULL)
	{
		if (dp->d_name[0]=='.' && !opa)
			;
		else
		{
			sprintf(name, "%s/%s", dirName, dp->d_name);
			listFile(name, name+strlen(dirName)+1);
		}
	}
	if (opc)
		printf("\n");
	return 0;
}

int main(int argc, char *argv[])
{
	int status=0;
	char opt[32];
	int wOpt=0;
	int i;

	if (argc > 1)
	{
		strcpy(opt, argv[1]);
		if (opt[0] == '-')
		{
			wOpt=1;
			for (i=1; i<strlen(opt); i++)
			{
				switch (opt[i])
				{
				case 'c':
					opc=1;
					opl=0;
					op1=0;
					break;
				case 'l':
					opc=0;
					opl=1;
					op1=0;
					break;
				case '1':
					opc=0;
					opl=0;
					op1=1;
					break;
				case 'a':
					opa=1;
					break;
				default:
					usage();
				}
			}
		}
	}

	if ((argc-wOpt)==1)
		status=listDir(".");
	else
	{
		if (wOpt == 1)
			*++argv;
		while (--argc > wOpt)
		{
			++argv;
			if (ftype(*argv) == 'd')
				status+=listDir(*argv);
			else
				status+=listFile(*argv, *argv);
		}
		if (opc)
			printf("\n");
	}
	return 0;
}
