/*
	mv.c: move files and directories.
	usage: mv [-fi] source1 [sourceN] target
	Ver. 0.5
	exit status =0 OK
		    >0 Error(s).
	-f Do not prompt for confirmation if the destination path exists.
	-i Prompt for confirmation if the destination path exists.
*/

#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include "bcom.h"
#define MAX_LEN	256

extern int errno;
char *prog="mv";
int opf=1;
int opi=0;
char overWrite='y';
char filler[MAX_LEN];

void usage()
{
	fprintf(stderr,"usage: %s [-fi] name1 [name2 ... nameN] target\n",
		prog);
	exit(1);
}

int cpFile(char *s1, char *s2)
{
	FILE *fi, *fo;
	char s3[MAX_LEN];
	int state=0, c;

	if (ftype(s2) == 'd')
		sprintf(s3,"%s/%s", s2, s1);
	else
		strcpy(s3, s2);
	if (fino(s1)==fino(s3))
	{
		fprintf(stderr,"%s: %s and %s are the same file, can't copy\n",
			prog, s1, s3);
		return 1;
	}

	if (fi=fopen(s1, "rb"))
	{
		if (opi && ftype(s3)!='n')
		{
			overWrite='x';
			while (overWrite!='y' && overWrite!='n')
			{
				printf("%s: overwrite %s? (y/n) ",
					prog, s3);
				overWrite=getc(stdin);
				gets(filler);
				fflush(stdin);
			}
			if (overWrite=='n')
			{
				fclose(fi);
				return 1;
			}
		}
		if (fo=fopen(s3, "wb"))
		{
			while ((c=fgetc(fi)) != EOF)
			{
				fputc(c, fo);
			}
			fclose(fo);
		}
		else
		{
			printf("%s: error %d writing %s\n", prog, errno, s3);
			state=1;
		}
		fclose(fi);
	}
	else
	{
		printf("%s: error %d reading %s\n", prog, errno, s1);
		state=1;
	}
	return(state);
}

int mvFile(char *s1, char *s2)
{
	int state=0, tmpState=0;

	if (ftype(s2)=='n')
	{
		tmpState=rename(s1, s2);
	}
	else
	{
		tmpState=cpFile(s1, s2);
		if (tmpState == 0)
			unlink(s1);
	}
	return(tmpState);
}

int mvDir(char *s1, char *s2)
{
	struct dirent *dp;
	DIR *dfd;
	char cName[MAX_LEN], dName[MAX_LEN];
	int state=0;

	if (ftype(s2) == 'n')
	{
		state=rename(s1, s2);
		return state;
	}

	if (ftype(s2) != 'd')
	{
		fprintf(stderr, "%s: Can't move %s to %s (isn't a dir).\n",
				prog, s1, s2);
		return 1;
	}

	sprintf(dName, "%s/%s", s2, s1);
	if (state=mkdir(dName, S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH) !=0)
	{
		fprintf(stderr, "%s: Error %d, can't create dir %s\n",
			prog, errno, dName);
		return state;
	}
	
	if ((dfd=opendir(s1)) == NULL)
	{
		fprintf(stderr,"%s Can't open %s\n", prog, s1);
		return 1;
	}
	while ((dp=readdir(dfd)) != NULL)
	{
		if (strcmp(dp->d_name,".")==0 || strcmp(dp->d_name,"..")==0)
			;
		else
		{
			printf("%s: found %s/%s\n", prog, s1, dp->d_name);
			sprintf(cName,"%s/%s",s1, dp->d_name);
			switch (ftype(cName))
			{
			case '-':
				state=state+mvFile(cName, s2);
				break;
			case 'd':
				state=state+mvDir(cName, s2);
				break;
			default:
				fprintf(stderr, "%s: Can't move %s.\n",
					prog, cName);
				++state;
			}
		}
	}
	closedir(dfd);

	if (rmdir(s1) != 0)
	{
		fprintf(stderr, "%s: Error %d, can't remove dir %s\n",
			prog, errno, s1);
	}
	return state;
}

int main(int argc, char *argv[])
{
	int i;
	char infile[MAX_LEN];
	char outfile[MAX_LEN];
	int status=0;
	char opt[32];
	int wOpt=0;

	if (argc < 3)
		usage();

	/* options check */
	strcpy(opt, argv[1]);
	if (opt[0]=='-')
	{
		wOpt=1;
		for (i=1; i<strlen(opt); i++)
		{
			switch (opt[i])
			{
			case 'i':
				opi=1;
				opf=0;
				break;
			case 'f':
				opf=1;
				opi=0;
				break;
			default:
				usage();
			}
		}
		if (argc < 3+wOpt)
			usage();
	}

	strcpy(outfile, argv[argc-1]);
	if (argc>(3+wOpt) && ftype(outfile)!='d')
	{
		fprintf(stderr,
			"%s: For more than 2 args, last must be a dir\n",
			prog);
		return(1);
	}

	for (i=(1+wOpt); i<(argc-1); i++)
	{
		strcpy(infile, argv[i]);
		switch (ftype(infile))
		{
		case '-':
			status = status + mvFile(infile, outfile);
			break;
		case 'd':
			status = status + mvDir(infile, outfile);
			break;
		default:
			fprintf(stderr, "%s: Can't move %s.\n",
				prog, infile);
			++status;
		}
	}

	return(status);
}
